// Object-Blinker an object that controls the (synchronous) blinking of other objects
function ObjectBlinker(sched)
{
	this.schedule = sched; // schedule according to which to show/hide objects from the list (regular array)
	this.list = [];        // list of objects to show/hide according to the schedule
	this.schedind = 0;      // where in the schedule are we
}

// a public method that determines whether the given object is in the list of blinkable objects
ObjectBlinker.prototype.find = function(obj)
{
	for (var ind in this.list)
	{
		if (this.list[ind].obj == obj)
			return ind;
	}

	return -1;
};

// a public method that removes the given object from the list of blinkable objects
ObjectBlinker.prototype.remove = function(obj)
{
	var ind = this.find(obj);

	if (ind >= 0)
	{
		// figure out whether to call onfun or offfun after removing the item from the list
		var fun = this.list[ind].end ? this.list[ind].on : this.list[ind].off;
		this.list.splice(ind, 1); // (because delete this.list[ind] does not update the length)
		fun(obj); // call onfun or offfun
	}
};

// a public method that adds an object to the list of blinkable objects
// the required parameters are:
// 1) obj---the object itself;
// 2) testfun---a function to be called (with obj as param) to test whether blinking obj should continue
// 3) endstate---boolean specifying whether upon removal from list obj should stay "on" or "off"
// 4) onfun---a function (called with obj as param) that "turns it on"
// 5) offfun---a function (called with obj as param) that "turns it off".
// endstate, onfun and offfun may be given as null, in which case they're interpreted as
// true, function(x) { x.show(); } and function(x) { x.hide(); }, respectively
ObjectBlinker.prototype.add = function(obj, testfun, endstate, onfun, offfun)
{
	var ind = this.find(obj);

	if (ind >= 0)
		return ind; // it's already in the list, just return its index

	if (endstate == null)
		endstate = true;

	if (onfun == null)
		onfun = function(x) { x.show(); };

	if (offfun == null)
		offfun = function(x) { x.hide(); };

	this.list.push({ obj:obj, test:testfun, end:endstate, on:onfun, off:offfun }); // add to the end

	if (this.schedind % 2 == 0) // even schedule index
		onfun(obj);
	else
		offfun(obj);

	if (this.list.length == 1) // it's the first object we're adding
		host.setTimeout(this.schedule[0], this.blinkAll, this);

	return this.list.length;
};

// this gets invoked from a new thread, blinks all the objects.
// (it is public, unfortunately, but trying to hide it seemed to involve having to make the
// other methods "privileged", which would basically make new closures of them for every
// new ObjectBlinker (not that we're planning to have too many of those, but still...))
ObjectBlinker.prototype.blinkAll = function()
{
	host.setTitle("Object Blinker");

	while (this.list.length > 0) // while we have objects in the list
	{
		if (++this.schedind >= this.schedule.length) // increment schedule index, with possible wraparound
			this.schedind = 0;

		var ind = 0;

		while (ind < this.list.length) // go through the list ((ind in list) does not survive index changes)
		{
			var item = this.list[ind]; // a list item---a 5-tuple of object, test function, end state, etc.

			if (item.test(item.obj)) // this object still tests as blinkable
			{
				if (this.schedind % 2 == 0) // even schedule index
					item.on(item.obj);
				else
					item.off(item.obj);
			}
			else // otherwise get rid of it in the list
			{
				var obj = this.list[ind].obj;
				var fun = this.list[ind].end ? this.list[ind].on : this.list[ind].off;
				this.list.splice(ind, 1); // this changes indices, adjusts the length
				fun(obj); // call onfun or offfun
				continue;
			}

			ind++;
		}

		host.sleep(this.schedule[this.schedind]);
	}
};

// this is the master object blinker that will act according to the usual schedule
var object_blinker = new ObjectBlinker([ 500, 500 ]); // 500 ms on, 500 ms off, repeat
//Currently only for Acces.Mod.
object_blinker_acc_mod = new ObjectBlinker([ 875, 250 ]);

